-- Reference to Pitbull 4
local PitBull4 = _G.PitBull4
if not PitBull4 then
	error("PitBull4_ThreatLine requires PitBull4")
end


-- Translations und new module
local Lpitbull 			= PitBull4.L
local Lmodule			= LibStub("AceLocale-3.0"):GetLocale("PitBull4_ThreatLine")
local PitBull4_ThreatLine 	= PitBull4:NewModule("TLBar", "AceEvent-3.0")

-- Local variables
local player_in_group = false			-- Is player in a group?
local ACCEPTABLE_CLASSIFICATIONS = {	-- Frames that are allowed to use this module
	player 		= true,
	pet 		= true,
	party 		= true,
	raid 		= true,
	partypet 	= true,
	raidpet 	= true,
	target 		= true,
	focus 		= true,
}
local tl_color = {						-- Default coloring
	[0] = {0.1, 0.8, 0.1, 1.0},
	[1] = {0.1, 0.4, 0.7, 1.0},			
	[2] = {0.8, 0.1, 0.1, 1.0},			
	[3] = {0.4, 0.4, 0.4, 1.0},			
}
local exclude = {						-- Excluded player (for default max threat holder search)
	["player"] = true,
}

-- ThreatLine bar module settings
PitBull4_ThreatLine:SetModuleType("bar")
PitBull4_ThreatLine:SetName(Lmodule["Threat Line"])
PitBull4_ThreatLine:SetDescription(Lmodule["Show a relative threat bar."])
PitBull4_ThreatLine:SetDefaults({
	size 		= 1.0,
	position 	= 5,
}, {
	tl_color 	= tl_color
})


-- Module enabled
function PitBull4_ThreatLine:OnEnable()
	-- Register events
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
	self:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("PLAYER_PET_CHANGED")
	
	-- Force update
	self:PARTY_MEMBERS_CHANGED()
end


-- Module disabled
function PitBull4_ThreatLine:OnDisable()
	-- Register events
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	self:UnregisterEvent("UNIT_THREAT_LIST_UPDATE")
	self:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	self:UnregisterEvent("RAID_ROSTER_UPDATE")
	self:UnregisterEvent("PLAYER_PET_CHANGED")
	
	-- Force update
	self:PARTY_MEMBERS_CHANGED()
end


-- Need to track thrat information?
function PitBull4_ThreatLine:PARTY_MEMBERS_CHANGED()
	-- In group?
	player_in_group = UnitExists("pet") or GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0
	
	-- Update all frames which are allowed to carry this module
	for classification in pairs(ACCEPTABLE_CLASSIFICATIONS) do
		self:UpdateForClassification(classification)
	end
end
-- Event forwarding
PitBull4_ThreatLine.RAID_ROSTER_UPDATE 				= PitBull4_ThreatLine.PARTY_MEMBERS_CHANGED
PitBull4_ThreatLine.PLAYER_PET_CHANGED 				= PitBull4_ThreatLine.PARTY_MEMBERS_CHANGED


-- Force update
function PitBull4_ThreatLine:PLAYER_TARGET_CHANGED()
	self:UpdateAll()
end
-- Event forwarding
PitBull4_ThreatLine.UNIT_THREAT_LIST_UPDATE 		= PitBull4_ThreatLine.PLAYER_TARGET_CHANGED
PitBull4_ThreatLine.UNIT_THREAT_SITUATION_UPDATE 	= PitBull4_ThreatLine.PLAYER_TARGET_CHANGED


-- Current bar value
function PitBull4_ThreatLine:GetValue(frame)
	-- Early exit
	if not frame or not ACCEPTABLE_CLASSIFICATIONS[frame.classification] or not player_in_group then return nil end
	
	-- Return value
	local threat = select(1, GetPlayerRelativeThreat(frame.unit))
	
	-- Only show if > 0
	if threat == 0 then
		return  nil
	else
		return threat / 100
	end
end
-- Example bar value
function PitBull4_ThreatLine:GetExampleValue(frame)
	-- Early exit
	if not frame or not ACCEPTABLE_CLASSIFICATIONS[frame.classification] then
		return nil
	end
	
	-- Return an example value
	return 0.25
end


-- Current bar color
function PitBull4_ThreatLine:GetColor(frame, value)
	-- Relative threat unit
	return GetPlayerTankedColor(frame.unit)
end
-- Example bar color
function PitBull4_ThreatLine:GetExampleColor(frame, value)
	-- Return an example value
	return self.db.profile.global.tl_color[0]
end


-- Add own color information
PitBull4_ThreatLine:SetColorOptionsFunction(function(self)
	-- Submenu entry
	return "tl_color0", 
	
	-- Options table
	{
		type = "color",
		name = Lmodule["Holding aggro"],
		desc = Lmodule["Which color to display when having aggro."],
		get = function(info)
			return unpack(self.db.profile.global.tl_color[0])
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.tl_color[0] = {r, g, b, a}
			self:UpdateAll()
		end,
	},
	
	"tl_color1", 
	{
		type = "color",
		name = Lmodule["Tank aggro"],
		desc = Lmodule["Which color to display when another tank has aggro."],
		get = function(info)
			return unpack(self.db.profile.global.tl_color[1])
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.tl_color[1] = {r, g, b, a}
			self:UpdateAll()
		end,
	},
	
	"tl_color2", 
	{
		type = "color",
		name = Lmodule["Non-tank aggro"],
		desc = Lmodule["Which color to display when someone else than a tank has aggro."],
		get = function(info)
			return unpack(self.db.profile.global.tl_color[2])
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.tl_color[2] = {r, g, b, a}
			self:UpdateAll()
		end,
	},
	
	"tl_color3", 
	{
		type = "color",
		name = Lmodule["Undefined aggro"],
		desc = Lmodule["Which color to display aggro status is unknown."],
		get = function(info)
			return unpack(self.db.profile.global.tl_color[3])
		end,
		set = function(info, r, g, b, a)
			self.db.profile.global.tl_color[3] = {r, g, b, a}
			self:UpdateAll()
		end,
	},
	
	-- Reset function
	function(info)
		self.db.profile.global.tl_color = tl_color
	end
end)


-- Return aggro color with raidtank support
function GetTankedColor(unitID, mobID)
	-- Early exit
	if not unitID or not mobID or not UnitExists(unitID) or not UnitExists(mobID) then return unpack(PitBull4_ThreatLine.db.profile.global.tl_color[3]) end

	-- No threat table possible
	if UnitIsFriend(unitID, mobID) or UnitIsPlayer(mobID) then return unpack(PitBull4_ThreatLine.db.profile.global.tl_color[3]) end

	-- Return aggro status
	local status = select(2, UnitDetailedThreatSituation(unitID, mobID))
	
	-- Wich status do we have?
	if not status then
		return unpack(PitBull4_ThreatLine.db.profile.global.tl_color[3])
	elseif status >= 2 then
		return unpack(PitBull4_ThreatLine.db.profile.global.tl_color[0])
	else
		-- Actually needed to do local maxID = select(1, GetMaxThreatUnit(mobID, exclude))
		maxID = mobID.."target"
			
		-- Current unit is in raid (and might be MT)
		if UnitInRaid(maxID) and select(10, GetRaidRosterInfo(maxID)) == "MAINTANK" then
			return unpack(PitBull4_ThreatLine.db.profile.global.tl_color[1])
		else
			return unpack(PitBull4_ThreatLine.db.profile.global.tl_color[2])
		end
	end
end


-- Return highest threat holder on mobIDs table (ignoring exclude units)
function GetMaxThreatUnit(mobID, exclude)
	-- Early exit #1
	if not UnitExists(mobID) or UnitIsFriend(mobID, "player") or UnitIsPlayer(mobID) then return end

	-- Local variables
	local group, size
	
	-- Get group type
	if GetNumRaidMembers() > 0 then 
		group 	= "raid"
		size 	= GetNumRaidMembers()
	elseif GetNumPartyMembers() > 0 then 
		group 	= "party"
		size 	= GetNumPartyMembers()
	else
		group	= nil
	end
	
	-- Early exit #3
	if not group and not UnitExists("pet") then return end
	
	-- Local variables
	local tempThreat, maxThreat, maxID = 0, 0, nil
	
	-- Add player if not unitID (compare to fixpoint)
	if not exclude or not exclude["player"] then
		tempThreat = select(3, UnitDetailedThreatSituation("player", mobID))
		
		-- Threat querried?
		if tempThreat and tempThreat > maxThreat then
			maxThreat	= tempThreat
			maxID 		= "player"
		end
	end
	
	-- Add pet if not unitID (compare to fixpoint)
	if UnitExists("pet") and (not exclude or not exclude["pet"]) then 
		-- Threat for the pet
		tempThreat	= select(3, UnitDetailedThreatSituation("pet", mobID))

		-- Threat querried?
		if tempThreat and tempThreat > maxThreat then
			maxThreat	= tempThreat
			maxID 		= "pet"
		end
	end

	-- Cycle through Group  (compare to fixpoint)
	if group then
		-- Local variables
		local index, groupID
	
		for index = 1, size do
			-- Threat of this party/raid-member
			groupID 	= group..index
			
			-- Ignore if they are the same
			if not exclude or not exclude[groupID] then
				tempThreat	= select(3, UnitDetailedThreatSituation(groupID, mobID))
				
				-- New highest threat member?
				if tempThreat and tempThreat > maxThreat then 
					maxThreat 	= tempThreat 
					maxID 		= groupID
				end
			end
	
			-- Threat for party/raid-member pet
			groupID 	= group..index.."pet"
			
			-- Ignore if they are the same
			if UnitExists(groupID) and (not exclude or not exclude[groupID]) then
				tempThreat	= select(3, UnitDetailedThreatSituation(groupID, mobID))
				
				-- New highest threat member?
				if tempThreat and tempThreat > maxThreat then 
					maxThreat 	= tempThreat 
					maxID 		= groupID
				end
			end
		end
	end
	
	-- Late exit :P
	return maxID, maxThreat
end
function GetMaxThreatNonPlayerUnit(mobID)
	return GetMaxThreatUnit(mobID, exclude)
end


-- Make sure to return an interesting mob watch threat on
-- UnitID is asumed to be frame.unit for the current frame
function GetPlayerRelativeID(unitID)
	-- Current (frame.)unit is the player (or empty)
	if not unitID or UnitIsUnit(unitID, "player")then
		return "target"
	end
	
	-- Unit"s target
	local targetID = unitID.."target"

	-- Current (frame.)unit"s target is the player (or empty)
	if  UnitIsUnit(targetID, "player") then
		return "target"
	else
		-- Current (frame.)unit is a mob and (frame.)unit"s target is a/the player
		if not UnitIsPlayer(unitID) and (UnitInRaid(targetID) or UnitInParty(targetID) or UnitIsUnit(targetID, "pet")) then
			return unitID
		-- Current (frame.)unit is a party/raid-member
		elseif UnitInRaid(unitID) or UnitInParty(unitID) or UnitIsUnit(unitID, "pet") then
			return targetID
		end
	end
	
	-- Default return
	return "target"
end


-- Smart LuaText relativ threat
-- unitID is assumed to be frame.unit
function GetPlayerRelativeThreat(unitID)
	-- Relative threat
	local mobID 			= GetPlayerRelativeID(unitID)
	local maxID, maxThreat	= GetMaxThreatUnit(mobID, exclude)
	
	-- Query players threat since we want relative
	local playerThreat = select(3, UnitDetailedThreatSituation("player", mobID)) or 0

	-- Maximum threat available
	if playerThreat then
		-- Player threat is available
		if maxThreat then
			-- How's first?
			if playerThreat > maxThreat then
				return math.abs(playerThreat - maxThreat), "player"
			else
				return math.abs(playerThreat - maxThreat), maxID
			end
		end

		return playerThreat, "player"
	elseif maxThreat then
		return maxThreat, maxID
	else
		return nil
	end
end


-- Smart LuaText aggro color with raidtank support
-- unitID is assumed to be frame.unit
function GetPlayerTankedColor(unitID)
	-- Relative mobID
	local mobID 	= GetPlayerRelativeID(unitID)
	
	-- Return
	return GetTankedColor("player", mobID)
end

